@@ -6,7 +6,7 @@
 # Creates a list of the data and other information needed for running the model
 getModelInputs <- function(
     data, outcome, obsID, pars , randPars, scalePar, randScale,
-    weights, panelID, clusterID, robust, startParBounds, startVals,
+    weights, panelID, clusterID, robust, startValBounds, startVals,
     numMultiStarts, useAnalyticGrad, scaleInputs, standardDraws, drawType,
     numDraws, numCores, vcov, predict, correlation, call, options
 ) {
@@ -23,7 +23,7 @@ getModelInputs <- function(
     panelID         = panelID,
     clusterID       = clusterID,
     robust          = robust,
-    startParBounds  = startParBounds,
+    startValBounds  = startValBounds,
     startVals       = startVals,
     numMultiStarts  = numMultiStarts,
     useAnalyticGrad = useAnalyticGrad,
@@ -50,21 +50,32 @@ getModelInputs <- function(
   # Get the design matrix, recoding parameters that are categorical
   # or have interactions
   recoded <- recodeData(data, pars, randPars)
+  formula <- recoded$formula
   X <- recoded$X
   pars <- recoded$pars
   randPars <- recoded$randPars
   scalePar <- defineScalePar(data, inputs, modelSpace)
   outcome <- as.matrix(data[outcome])
 
-  # Setup obsID
-  obsID <- makeObsID(data, obsID, outcome)
+  # Setup weights
+  weights <- matrix(1, nrow(data))
+  weightsUsed <- FALSE
+  if (!is.null(inputs$weights)) {
+      weights <- as.matrix(data[inputs$weights])
+      weightsUsed <- TRUE
+  }
 
-  # Setup panelID
+  # Setup IDs
   panel <- !is.null(inputs$panelID)
-  if (panel) {
-      panelID <- as.matrix(data[panelID])
-      reps <- as.numeric(table(panelID))
-      panelID <- rep(seq_along(reps), reps) # Make sure it's a sequential number
+  if (panel) { panelID <- makePanelID(data, inputs) }
+  obsID <- makeObsID(data, inputs, outcome)
+  inputs <- setupClusterID(inputs, panel, robust, weightsUsed)
+  if (!is.null(inputs$clusterID)) {
+      if (robust == FALSE) {
+          message("Setting robust to TRUE since clusters are being used")
+          inputs$robust <- TRUE
+      }
+      clusterID <- makeClusterID(data, inputs, obsID, panelID)
   }
 
   # Set up other objects defining aspects of model
@@ -84,8 +95,10 @@ getModelInputs <- function(
     draws       = numDraws,
     multiStarts = numMultiStarts,
     obs         = sum(outcome),
-    cores       = setNumCores(numCores) # cores for parallel processing
+    cores       = setNumCores(numCores), # cores for parallel processing
+    clusters    = 0
   )
+  if (!is.null(inputs$clusterID)) { n$clusters <- length(unique(clusterID)) }
   parNames <- getParNames(parSetup, n, correlation)
   n$pars <- length(parNames$all)
   parIDs <- getParIDs(parSetup, n, modelSpace, modelType, randScale,correlation)
@@ -96,26 +109,6 @@ getModelInputs <- function(
     inputs$startVals <- startVals
   }
 
-  # Setup weights
-  weights <- matrix(1, nrow(data))
-  weightsUsed <- FALSE
-  if (!is.null(inputs$weights)) {
-    weights <- as.matrix(data[inputs$weights])
-    weightsUsed <- TRUE
-  }
-
-  # Setup clusters
-  n$clusters <- 0
-  inputs <- setupClusters(inputs, panel, robust, weightsUsed)
-  if (!is.null(inputs$clusterID)) {
-    if (robust == FALSE) {
-      message("Setting robust to TRUE since clusters are being used")
-      inputs$robust <- TRUE
-    }
-    clusterID <- as.matrix(data[inputs$clusterID])
-    n$clusters <- length(unique(clusterID))
-  }
-
   # Make data object
   data <- list(
     scalePar  = scalePar,
@@ -154,6 +147,7 @@ getModelInputs <- function(
   # Make modelInputs list
   modelInputs <- list(
     call          = call,
+    formula       = formula,
     date          = format(Sys.time(), "%a %b %d %X %Y"),
     version       = as.character(utils::packageVersion("logitr")),
     inputs        = inputs,
@@ -192,19 +186,29 @@ getModelInputs <- function(
   return(modelInputs)
 }
 
-makeObsID <- function(data, obsID, outcome) {
-  obsID <- as.vector(as.matrix(data[obsID]))
+checkRepeatedIDs <- function(var, id, reps) {
+    idCheck <- as.numeric(rep(names(reps), as.numeric(reps)))
+    if (!all(id == idCheck)) {
+        stop("The '", var,"' variable provided has repeated ID values.")
+    }
+}
+
+makePanelID <- function(data, inputs) {
+    panelID <- as.vector(as.matrix(data[inputs$panelID]))
+    # Make sure panelID is in sequential order with no repeated IDs
+    reps <- table(panelID)
+    checkRepeatedIDs('panelID', panelID, reps)
+    # Passed all checks, so now create a sequentially increasing numeric
+    # sequence for the panelID
+    panelID <- rep(seq_along(reps), reps)
+    return(panelID)
+}
+
+makeObsID <- function(data, inputs, outcome) {
+  obsID <- as.vector(as.matrix(data[inputs$obsID]))
   # Make sure obsID is in sequential order with no repeated IDs
   reps <- table(obsID)
-  obsIDCheck <- as.numeric(rep(names(reps), as.numeric(reps)))
-  if (!all(obsID == obsIDCheck)) {
-    stop(
-      "The 'obsID' variable provided has repeated ID values. It must be a ",
-      "sequentially increasing numeric value identifying the rows of each ",
-      "observation, such as 1,1,1,2,2,2...double check the variable provided ",
-      "for 'obsID'."
-    )
-  }
+  checkRepeatedIDs('obsID', obsID, reps)
   # Make sure that each observation ID has only one outcome
   outcomeCheck <- tapply(outcome, obsID, function(x) sum(x))
   if (! all(outcomeCheck == 1)) {
@@ -214,9 +218,61 @@ makeObsID <- function(data, obsID, outcome) {
       "the variables provided for 'obsID' and 'outcome'."
     )
   }
+  # Passed all checks, so now create a sequentially increasing numeric
+  # sequence for the obsID
+  obsID <- rep(seq_along(reps), reps)
   return(obsID)
 }
 
+setupClusterID <- function(inputs, panel, robust, weightsUsed) {
+    if (panel & robust) {
+        if (!identical(inputs$clusterID, inputs$panelID)) {
+            message(
+                "Setting clusterID to '", inputs$panelID, "' since robust == TRUE ",
+                "and a panelID is provided")
+            inputs$clusterID <- inputs$panelID
+        }
+    }
+
+    if (robust & is.null(inputs$clusterID)) {
+        message(
+            "Setting clusterID to '", inputs$obsID, "' since robust == TRUE ")
+        inputs$clusterID <- inputs$obsID
+    }
+
+    if (weightsUsed & is.null(inputs$clusterID)) {
+        if (panel) {
+            message(
+                "Setting clusterID to '", inputs$panelID, "' since weights are being ",
+                "used and no clusterID was provided")
+            inputs$clusterID <- inputs$panelID
+        } else {
+            message(
+                "Setting clusterID to '", inputs$obsID, "' since weights are being ",
+                "used and no clusterID was provided")
+            inputs$clusterID <- inputs$obsID
+        }
+    }
+    return(inputs)
+}
+
+makeClusterID <- function(data, inputs, obsID, panelID) {
+    if (inputs$clusterID == inputs$obsID) {
+        return(obsID)
+    }
+    if (inputs$clusterID == inputs$panelID) {
+        return(panelID)
+    }
+    clusterID <- as.vector(as.matrix(data[inputs$clusterID]))
+    # Make sure clusterID is in sequential order with no repeated IDs
+    reps <- table(clusterID)
+    checkRepeatedIDs('clusterID', clusterID, reps)
+    # Passed all checks, so now create a sequentially increasing numeric
+    # sequence for the clusterID
+    clusterID <- rep(seq_along(reps), reps)
+    return(clusterID)
+}
+
 setNumCores <- function(numCores) {
   coresAvailable <- parallel::detectCores()
   maxCores <- coresAvailable - 1
@@ -254,7 +310,7 @@ defineScalePar <- function(data, inputs, modelSpace) {
     if (! typeof(scalePar) %in% c("integer", "double")) {
       stop(
         'Please make sure the "scalePar" column in your data ',
-        'is encoded as a numeric data type. Scale must ',
+        'is encoded as a numeric data type. The scalePar must ',
         'be numeric and continuous.'
       )
     }
@@ -372,38 +428,6 @@ getPartialIDs <- function(parIDs, IDs, n, correlation) {
   return(partial_IDs)
 }
 
-setupClusters <- function(inputs, panel, robust, weightsUsed) {
-  if (panel & robust) {
-    if (!identical(inputs$clusterID, inputs$panelID)) {
-      message(
-        "Setting clusterID to '", inputs$panelID, "' since robust == TRUE ",
-        "and a panelID is provided")
-      inputs$clusterID <- inputs$panelID
-    }
-  }
-
-  if (robust & is.null(inputs$clusterID)) {
-    message(
-      "Setting clusterID to '", inputs$obsID, "' since robust == TRUE ")
-    inputs$clusterID <- inputs$obsID
-  }
-
-  if (weightsUsed & is.null(inputs$clusterID)) {
-    if (panel) {
-      message(
-        "Setting clusterID to '", inputs$panelID, "' since weights are being ",
-        "used and no clusterID was provided")
-      inputs$clusterID <- inputs$panelID
-    } else {
-      message(
-        "Setting clusterID to '", inputs$obsID, "' since weights are being ",
-        "used and no clusterID was provided")
-      inputs$clusterID <- inputs$obsID
-    }
-  }
-  return(inputs)
-}
-
 getScaleFactors <- function(
   data, modelSpace, modelType, parIDs, parNames, n, correlation
 ) {
